#include "itogmassive.h"
#include "QTime"
#include "qmath.h"

ItogMassiv::ItogMassiv()
{
    Tatmos=0;
    X=-1;
    Y=-1;
    shag=0;

    popravka1=0;
    popravka2=0;
}

ItogMassiv::~ItogMassiv()
{
    //проверка была ли удалена матрица раньше
    if (X!=-1&&Y!=-1) this->deleteMatrica();
}

void ItogMassiv::createMacroMatrica(int yvelichenie,
                                    int vert_coef,
                                    int gor_coef)
{

    //увеличение матрицы не может быть меньше 1
    if (yvelichenie<1) return;
    //количество элементов тоже
    if (vert_coef<1) return;
    if (gor_coef<1) return;




    //форматируем матрицу, если новый размер не совпал
    if (X!=4*yvelichenie*vert_coef ||
            Y!=16*yvelichenie*gor_coef)
    {
        shag = yvelichenie;

        //проверка была ли создана матрица раньше
        if (X!=-1&&Y!=-1) this->deleteMatrica();


        X=4*shag*vert_coef;
        Y=16*shag*gor_coef;

        itog_mas=new double *[X];
        for (int i=0;i<X;i++)
            itog_mas[i]=new double [Y];
    }

    //заполняем массив нулями
    for (int j=0;j<Y;j++)
        for (int i=0;i<X;i++)
        {
            itog_mas[i][j]=0;
        }
}

void ItogMassiv::deleteMatrica()
{
    for (int i=0;i<X;i++)
        delete [] itog_mas[i];
    delete [] itog_mas;
    X=-1;
    Y=-1;
}

void ItogMassiv::setRandom()
{
    QTime midnight(0,0,0);
    qsrand(midnight.secsTo(QTime::currentTime()));

    for (int j=0;j<Y;j++)
        for (int i=0;i<X;i++)
        {
            itog_mas[i][j]=(qrand() /(qPow(10, 2) + 1))/qPow(10, 2)*3+25;
        }
}


void ItogMassiv::setMatica(QSerialPort *CommandPort,
                           int yvelichenie,
                           int micro_payza,
                           int micro_shag,
                           int macro_payza,
                           int nachalo_vertical,
                           int kolichestvo_vertical,
                           int shag_vertical,
                           int nachalo_gorizontal,
                           int kolichestvo_gorizontal,
                           int shag_gorizontal,
                           int TempObj,
                           int TempAtmos,
                           int compOscTrim)
{
    //Формируем строку с конфигурацией системы
    QString command_to_termoscanner="day matricy "+
            QString::number(yvelichenie)+" "+
            QString::number(micro_payza)+" "+
            QString::number(micro_shag)+" "+
            QString::number(macro_payza)+" "+
            QString::number(nachalo_vertical)+" "+
            QString::number(kolichestvo_vertical)+" "+
            QString::number(shag_vertical)+" "+
            QString::number(nachalo_gorizontal)+" "+
            QString::number(kolichestvo_gorizontal)+" "+
            QString::number(shag_gorizontal)+" "+
            QString::number(TempObj)+" "+
            QString::number(TempAtmos)+" "+
            QString::number(compOscTrim);



    //Передаем конфигурацию
    CommandPort->write(command_to_termoscanner.toLatin1());

    double buf[1];
    CommandPort->setReadBufferSize(sizeof(buf)*X*Y+sizeof(buf));
    CommandPort->clear(QSerialPort::Input);

    bool kik=true;
    bool poexali=false;;
    int ch=0;
    while (kik)
    {
        if (CommandPort->waitForReadyRead(100))
        {
            if (!poexali) poexali=true;

            if (CommandPort->bytesAvailable()==sizeof(buf)*X*Y+sizeof(buf))
            {
                for (int i=0;i<X;i++)
                {
                    for (int j=0;j<Y;j++)
                    {
                        CommandPort->read((char*)buf,sizeof(buf));
                        itog_mas[i][j]=buf[0];
                    }
                }
                CommandPort->read((char*)buf,sizeof(buf));
                Tatmos=buf[0];

                kik=false;
            }
        }
        if (poexali&&(ch==CommandPort->bytesAvailable()))
        {
            QString yvedomlenie="1";
            CommandPort->write(yvedomlenie.toLatin1());
            poexali=false;
            ch=0;
        }
        ch=CommandPort->bytesAvailable();
    }
    QString yvedomlenie="0";
    CommandPort->write(yvedomlenie.toLatin1());

}
