#include "mainwindow.h"
#include "ui_mainwindow.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    cur="";

    log.setFileName("termoscan.log");
    log.open(QIODevice::WriteOnly);

    fake_COM=false;
    COM="";
    CommandPort=new QSerialPort;
    //получение начальной конфигурации портов
    this->refresh_COM_ports();
    //получение доступных скоростей портов
    this->get_speed_COM_ports();



    termo_shkala=new QGraphicsScene;
    termo_matrica=new VisualTermo;
    //сбрасываем значения, чтобы они не остались,
    //если ткнули в области отображния
    termo_matrica->mouse_on_pixel_pos.setX(-1);
    termo_matrica->mouse_on_pixel_pos.setY(-1);


    stroim_radygy(termo_shkala);
    zadaem_chisla_na_shkale(termo_shkala,ui->horizontalSlider->value(),
                            ui->horizontalSlider_2->value());
    ui->graphicsView_2->setScene(termo_shkala);
    ui->graphicsView_2->setRenderHint(QPainter::Antialiasing, true);

    itog_mas=new ItogMassiv;

    ui->horizontalSlider_7->setSingleStep(ui->lineEdit_3->text().toInt());
    ui->horizontalSlider_7->setPageStep(ui->lineEdit_3->text().toInt()*10);
}

MainWindow::~MainWindow()
{
    delete CommandPort;
    delete itog_mas;
    delete termo_matrica;
    delete termo_shkala;
    delete ui;
}


void MainWindow::on_pushButton_clicked()
{

    if (CommandPort->error()!=QSerialPort::NoError&&
            CommandPort->error()!=QSerialPort::TimeoutError)
    {
        real_data=QDateTime::currentDateTime();
        _COMChat=_COMChat+real_data.toString("dd.MM.yy [hh:mm:ss.zzz]: ")+
                "Ошибка передачи данных №"+QString::number(CommandPort->error())+
                ". Попробуйте перезагрузить устройство.\r\n";
        ui->chatWithCOMTextBrowser->setText(_COMChat);
        ui->chatWithCOMTextBrowser->verticalScrollBar()->
                setValue(ui->chatWithCOMTextBrowser->verticalScrollBar()->maximum());

        log.close();
        log.open(QIODevice::WriteOnly);
        log.write(_COMChat.toLocal8Bit());
        log.close();
        log.open(QIODevice::Append);
        return;
    }

    ui->widget->setEnabled(false);
    ui->widget_2->setEnabled(false);
    ui->widget_3->setEnabled(false);
    ui->pushButton_4->setEnabled(true);





    itog_mas->createMacroMatrica(ui->lineEdit->text().toInt(),
                                 ui->lineEdit_8->text().toInt(),
                                 ui->lineEdit_5->text().toInt());


    real_data=QDateTime::currentDateTime();
    _COMChat=_COMChat+real_data.toString("dd.MM.yy [hh:mm:ss.zzz]: ")+"Запрос изображения.\r\n";
    ui->chatWithCOMTextBrowser->setText(_COMChat);
    ui->chatWithCOMTextBrowser->verticalScrollBar()->
            setValue(ui->chatWithCOMTextBrowser->verticalScrollBar()->maximum());

    log.close();
    log.open(QIODevice::WriteOnly);
    log.write(_COMChat.toLocal8Bit());
    log.close();
    log.open(QIODevice::Append);

    int freq_obj=8;
    switch (ui->comboBox_2->currentIndex())
    {
    case 0:
        freq_obj=15;
        break;
    case 1:
        freq_obj=14;
        break;
    case 2:
        freq_obj=13;
        break;
    case 3:
        freq_obj=12;
        break;
    case 4:
        freq_obj=11;
        break;
    case 5:
        freq_obj=10;
        break;
    case 6:
        freq_obj=9;
        break;
    case 7:
        freq_obj=8;
        break;
    case 8:
        freq_obj=7;
        break;
    case 9:
        freq_obj=6;
        break;
    case 10:
        freq_obj=5;
        break;
    }

    int freq_atmos=116;
    switch (ui->comboBox_2->currentIndex())
    {
    case 0:
        freq_atmos=116;
        break;
    case 1:
        freq_atmos=100;
        break;
    case 2:
        freq_atmos=84;
        break;
    case 3:
        freq_atmos=68;
        break;
    }

    if (ui->radioButton->isChecked())
        itog_mas->setMatica(CommandPort,
                            ui->lineEdit->text().toInt(),
                            ui->lineEdit_2->text().toInt(),
                            ui->lineEdit_3->text().toInt(),
                            ui->lineEdit_13->text().toInt(),
                            ui->lineEdit_7->text().toInt(),
                            ui->lineEdit_8->text().toInt(),
                            ui->lineEdit_9->text().toInt(),
                            ui->lineEdit_6->text().toInt(),
                            ui->lineEdit_5->text().toInt(),
                            ui->lineEdit_4->text().toInt(),
                            freq_obj,
                            freq_atmos,
                            ui->spinBox->value());


    if (ui->radioButton_3->isChecked()) itog_mas->setRandom();

    ui->label_10->setText("<html><head/><body><p><span style=' font-weight:600;'>Окружающая среда: "+
                          QString::number(itog_mas->getTatmos(),'g',4)+"°С</span></p></body></html>");

    stroim_kartinky(itog_mas,termo_matrica,ui->horizontalSlider->value(),
                    ui->horizontalSlider_2->value());
    ui->graphicsView->setScene(termo_matrica);
    termo_matrica->setSceneRect(termo_matrica->itemsBoundingRect());
    ui->graphicsView->fitInView(termo_matrica->sceneRect(),Qt::KeepAspectRatio);



    double max_temp=itog_mas->getTemp(0,0);
    double min_temp=itog_mas->getTemp(0,0);
    for (int i=0;i<itog_mas->x();i++)
    {
        for (int j=0;j<itog_mas->y();j++)
        {
            if (itog_mas->getTemp(i,j)>max_temp) max_temp=itog_mas->getTemp(i,j);
            if (itog_mas->getTemp(i,j)<min_temp) min_temp=itog_mas->getTemp(i,j);
        }
    }
    ui->label_26->setText("Мин: "+QString::number(min_temp,'g',4)+"°С");
    ui->label_32->setText("Макс: "+QString::number(max_temp,'g',4)+"°С");

    if (ui->checkBox_2->isChecked())
    {
        ui->horizontalSlider->setValue(min_temp);
        ui->horizontalSlider_2->setValue(max_temp);
    }

    real_data=QDateTime::currentDateTime();
    _COMChat=_COMChat+real_data.toString("dd.MM.yy [hh:mm:ss.zzz]: ")+"Изображение получено.\r\n";
    ui->chatWithCOMTextBrowser->setText(_COMChat);
    ui->chatWithCOMTextBrowser->verticalScrollBar()->
            setValue(ui->chatWithCOMTextBrowser->verticalScrollBar()->maximum());

    log.close();
    log.open(QIODevice::WriteOnly);
    log.write(_COMChat.toLocal8Bit());
    log.close();
    log.open(QIODevice::Append);


    ui->widget->setEnabled(true);
    ui->widget_2->setEnabled(true);
    ui->widget_3->setEnabled(true);
}

void MainWindow::on_pushButton_5_clicked()
{
    ui->stackedWidget->setCurrentIndex(1);
}

void MainWindow::on_pushButton_6_clicked()
{
    ui->stackedWidget->setCurrentIndex(0);
}

void MainWindow::on_horizontalSlider_7_valueChanged(int )
{
    ui->label_25->setText(QString::number(ui->horizontalSlider_7->value())+" мкс");
}

void MainWindow::on_lineEdit_3_editingFinished()
{
    ui->horizontalSlider_7->setSingleStep(ui->lineEdit_3->text().toInt());
    ui->horizontalSlider_7->setPageStep(ui->lineEdit_3->text().toInt()*10);
}

void MainWindow::on_checkBox_2_clicked(bool checked)
{
    if (checked)
    {
        ui->checkBox->setChecked(false);
        ui->horizontalSlider_3->setEnabled(false);
        ui->label_3->setEnabled(false);
        ui->checkBox->setEnabled(false);
    }
    else ui->checkBox->setEnabled(true);
}

void MainWindow::on_pushButton_4_clicked()
{
    QDateTime real_vremya;
    real_vremya=QDateTime::currentDateTime();

    cur.remove(cur.lastIndexOf("/")+1,cur.length()-cur.lastIndexOf("/"));
    cur=QFileDialog::getSaveFileName(this,"Сохранение изображения термограммы",cur+
                                     "Термограмма"+real_vremya.toString(" dd.MM.yy [hh-mm-ss]"),
                                     "BMP (*.bmp);;JPG (*.jpg);;JPEG (*.jpeg);;PNG (*.png)");

    if(cur!="")
    {
        int tmw = termo_matrica->width();
        int tmh = termo_matrica->height();
        int shw = termo_shkala->width();
        int shh = termo_shkala->height();

        int height=1050;
        int width=1400;

        QImage buf_image(width,height,QImage::Format_ARGB32_Premultiplied);
        buf_image.fill(Qt::white);
        QPainter painter(&buf_image);
        painter.setRenderHint(QPainter::HighQualityAntialiasing);



        if ((double)width/tmw>=(double)height/tmh)
        {
            termo_matrica->render(&painter,
                                  QRectF(width/2-(tmw*(height-shh+80)/tmh)/2+12,0,width,height-shh+80),
                                  QRectF(0,0,tmw,tmh),
                                  Qt::KeepAspectRatio);
        }
        else
        {
            termo_matrica->render(&painter,
                                  QRectF(0,(height-shh+80)/2-(tmh*width/tmw)/2+12,width,height-shh+80),
                                  QRectF(0,0,tmw,tmh),
                                  Qt::KeepAspectRatio);
        }


        termo_shkala->render(&painter,
                             QRectF(5,height-shh+100,width-600,height),
                             QRectF(-63,-5,shw+10,shh-50),
                             Qt::KeepAspectRatio);


        painter.setFont(QFont("Inconsolata",14,QFont::Bold));
        painter.drawText(850,height-shh+180,"Окружающая среда: "+QString::number(itog_mas->getTatmos(),'g',4)+"°С");
        painter.drawText(850,height-shh+210,ui->label_26->text());
        painter.drawText(1000,height-shh+210,ui->label_32->text());
        painter.drawText(1240,height-shh+180,real_vremya.toString("dd.MM.yy"));
        painter.drawText(1210,height-shh+210,real_vremya.toString("[hh:mm:ss.zzz]"));


        painter.setFont(QFont("Inconsolata",19,QFont::Bold,QFont::Cursive));
        QString buf_cur = cur;
        buf_cur.remove(0,cur.lastIndexOf("/")+1);

        if (buf_cur.length()>35)
        {
            QString buf=buf_cur;
            QString itog_buf="";

            buf.remove(16,buf.length());
            itog_buf+=buf+"...";
            buf=buf_cur;
            buf.remove(0,buf.length()-16);
            itog_buf+=buf;
            buf_cur=itog_buf;
        }

        painter.drawText(825,height-shh+110,550,30,Qt::AlignCenter,"«"+buf_cur+"»");


        buf_image.save(cur);

        real_data=QDateTime::currentDateTime();
        _COMChat=_COMChat+real_data.toString("dd.MM.yy [hh:mm:ss.zzz]: ")+"Изображение сохранено.\r\n"+cur+"\r\n";
        ui->chatWithCOMTextBrowser->setText(_COMChat);
        ui->chatWithCOMTextBrowser->verticalScrollBar()->
                setValue(ui->chatWithCOMTextBrowser->verticalScrollBar()->maximum());

        log.close();
        log.open(QIODevice::WriteOnly);
        log.write(_COMChat.toLocal8Bit());
        log.close();
        log.open(QIODevice::Append);
    }
}
