#include "visualtermo.h"

#include <QGraphicsItem>
#include <QGraphicsSceneMouseEvent>



void stroim_radygy(QGraphicsScene* graphicsScene2)
{
    graphicsScene2->clear();
    //задаем размер изображения, необходимый для шкалы
    QImage image(1536,100,QImage::Format_ARGB32_Premultiplied);
    //заполняем прозрачными пикселями, чтобы не было мусорных цветов
    image.fill(0x00000000);

    //строим радугу из 1280 значений
    for (int i=0; i<1536; i++)
    {
        unsigned int pixel=0;
        //переключатель для перехода цветов в радуге по коду цвета
        if (1280<=i && i<1536) pixel=0xFFFF0000+256*(i-1280)+(i-1280);
        if (1024<=i && i<1280) pixel=0xFFFFFF00-256*(i-1024);
        if (768<=i && i<1024) pixel=0xFF00FF00+256*256*(i-768);
        if (512<=i && i<768) pixel=0xFF00FFFF-(i-512);
        if (256<=i && i<512) pixel=0xFF0000FF+256*(i-256);
        if (0<=i && i<256) pixel=0xFFFF00FF-256*256*i;

        //задаем ширину радуги
        for (int j=0; j<100; j++) image.setPixel(i,j,pixel);
    }

    //добавляем радугу на сцену
    QPixmap pixmap;
    graphicsScene2->addPixmap(pixmap.fromImage(image));

    //добавляем черную рамку для радуги на сцену
    graphicsScene2->addRect(0,0,1536,100,QPen(Qt::black, 4,  Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));


    //добавляем шкалу-черточки 13 штук, разбитие на 12 равных частей
    //3 большие, 8 малые, без цифровых значений
    //малые
    for (int i=128; i<768; i=i+128)
        graphicsScene2->addLine(0+i,120,0+i,150,QPen(Qt::black, 3,  Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin));
    for (int i=896; i<1536; i=i+128)
        graphicsScene2->addLine(0+i,120,0+i,150,QPen(Qt::black, 3,  Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin));
    //большие
    for (int i=0; i<=1536; i=i+768)
        graphicsScene2->addLine(0+i,120,0+i,180,QPen(Qt::black, 5,  Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin));

    //рисуем символ температуры
    QGraphicsTextItem *pic_text;
    pic_text=graphicsScene2->addText("°С",QFont("Calibri",58,QFont::Bold));
    pic_text->setPos(1551,0);
}

void zadaem_chisla_na_shkale(QGraphicsScene *graphicsScene2,
                             int V_min, int V_max)
{
    //удаляем старые числа шкалы
    QList<QGraphicsItem*> list = graphicsScene2->items();

    //если они есть
    if (list.size()>=29)
    {
        for (int i=0; i<13; i++)
        {
            graphicsScene2->removeItem(list.at(i));
        }
    }

    //добавляем новые числа
    QGraphicsTextItem *pic_text;
    double dolya=(double)(V_max-V_min)/12;
    QString chislo_print;
    int calibrovochnoe=0; //центрируем относительно длины числа
    //2-4 числа
    for (int i=128; i<768; i=i+128)
    {
        double chislo=V_min+dolya*i/128;
                //проверяем наличие запятой для центрирования
        if (chislo - (int)chislo ==0 ) calibrovochnoe=13;
        else calibrovochnoe=9;
        chislo_print=QString::number(chislo,'g',4);
        pic_text=graphicsScene2->addText(chislo_print,QFont("Inconsolata",25,QFont::Normal));
        pic_text->setPos(-chislo_print.size()*calibrovochnoe+i,150);
    }
    //6-9 числа
    for (int i=896; i<1536; i=i+128)
    {
        double chislo=V_min+dolya*i/128;
                //проверяем наличие запятой для центрирования
        if (chislo - (int)chislo ==0 ) calibrovochnoe=13;
        else calibrovochnoe=9;
        chislo_print=QString::number(V_min+dolya*i/128,'g',4);
        pic_text=graphicsScene2->addText(chislo_print,QFont("Inconsolata",25,QFont::Normal));
        pic_text->setPos(-chislo_print.size()*calibrovochnoe+i,150);
    }
    //1,5,10 числа
    for (int i=0; i<=1536; i=i+768)
    {
        double chislo=V_min+dolya*i/128;
        //проверяем наличие запятой для центрирования
        if (chislo - (int)chislo ==0 ) calibrovochnoe=17;
        else calibrovochnoe=14;
        //добавляем знак больше, меньше
        if (i==0) chislo_print="<"+QString::number(V_min+dolya*i/128,'g',4);;
        if (i==1536) chislo_print=QString::number(V_min+dolya*i/128,'g',4)+"<";
        if (i!=0&&i!=1536) chislo_print=QString::number(V_min+dolya*i/128,'g',4);


        pic_text=graphicsScene2->addText(chislo_print,QFont("Inconsolata",30,QFont::Bold));
        pic_text->setPos(-chislo_print.size()*calibrovochnoe+i,180);
    }
}

void stroim_kartinky(ItogMassiv* itog_mas,
QGraphicsScene *graphicsScene, int V_min, int V_max)
{
    graphicsScene->clear();
    //определяем множитель для отображения температур на множество от 0 до 1536
    double coef=(double)1536/(V_max-V_min);
    QImage image(itog_mas->y(), itog_mas->x(), QImage::Format_ARGB32_Premultiplied);
    //если температура выше диапозона, то фиолетовый цвет
    image.fill(0xFFFF00FF);


    int chislo=0;
    for (int j=0;j<itog_mas->y();j++)
    {
        for (int i=0;i<itog_mas->x();i++)
        {
            if (V_min<=itog_mas->getTemp(i,j) && itog_mas->getTemp(i,j)<=V_max)
            {
                //преобразуем температуру до вида от 0 до 1536
                chislo=(itog_mas->getTemp(i,j)-V_min)*coef+0.05;
                unsigned int pixel=0;
                if (1280<=chislo && chislo<1536) pixel=0xFFFF0000+256*(chislo-1280)+(chislo-1280);
                if (1024<=chislo && chislo<1280) pixel=0xFFFFFF00-256*(chislo-1024);
                if (768<=chislo && chislo<1024) pixel=0xFF00FF00+256*256*(chislo-768);
                if (512<=chislo && chislo<768) pixel=0xFF00FFFF-(chislo-512);
                if (256<=chislo && chislo<512) pixel=0xFF0000FF+256*(chislo-256);
                if (0<=chislo && chislo<256) pixel=0xFFFF00FF-256*256*chislo;

                image.setPixel(j,i,pixel);
            }

            //если температура выше диапозона, то белый цвет
            if (itog_mas->getTemp(i,j)>V_max)
            {
                //преобразуем температуру до вида от 0 до 1280
                chislo=(itog_mas->getTemp(i,j)-V_min)*coef+0.05;
                image.setPixel(j,i,0xFFFFFFFF);
            }
        }
    }


    QPixmap pixmap;
    graphicsScene->addPixmap(pixmap.fromImage(image));
}


void VisualTermo::mousePressEvent(QGraphicsSceneMouseEvent * mouse_press_event)
{
    //определяем координаты клика по матрице температур
    mouse_on_pixel_pos=mouse_press_event->scenePos();
}
void VisualTermo::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent )
{
    mouseEvent->ignore();
}
